# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "ChatCompletionCreateParamsBase",
    "Message",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessage",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsTextUserMessageContentParts",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentParts",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentPartsImageURL",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentParts",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentPartsImageData",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsAssistantMessage",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsSystemMessage",
    "MemoryStrategy",
    "MemoryStrategyParams",
    "ModelParameters",
    "ChatCompletionCreateParamsNonStreaming",
    "ChatCompletionCreateParamsStreaming",
]


class ChatCompletionCreateParamsBase(TypedDict, total=False):
    messages: Required[Iterable[Message]]
    """The list of messages in the conversation.

    Expand each message type to see how it works and when to use it. Most
    conversations should begin with a single `user` message.
    """

    model: Required[
        Literal[
            "gpt-4",
            "gpt-4-0613",
            "gpt-4-32k",
            "gpt-4-32k-0613",
            "gpt-4o",
            "gpt-4o-2024-08-06",
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-0613",
            "gpt-3.5-turbo-16k",
            "gpt-3.5-turbo-16k-0613",
            "gemini-pro",
            "gemini-1.5-pro-001",
            "gemini-1.5-pro-preview-0409",
            "gemini-1.5-pro-preview-0514",
            "llama-2-7b-chat",
            "llama-2-13b-chat",
            "llama-2-70b-chat",
            "llama-3-8b-instruct",
            "llama-3-70b-instruct",
            "llama-3-1-8b-instruct",
            "llama-3-1-70b-instruct",
            "Meta-Llama-3-8B-Instruct-RMU",
            "Meta-Llama-3-8B-Instruct-RR",
            "Meta-Llama-3-8B-Instruct-DERTA",
            "Meta-Llama-3-8B-Instruct-LAT",
            "mixtral-8x7b-instruct",
            "mixtral-8x22b-instruct",
            "claude-3-opus-20240229",
            "claude-3-sonnet-20240229",
            "claude-3-haiku-20240307",
            "claude-3-5-sonnet-20240620",
            "mistral-large-latest",
            "phi-3-mini-4k-instruct",
            "phi-3-cat-merged",
            "zephyr-cat-merged",
            "dolphin-2.9-llama3-8b",
            "dolphin-2.9-llama3-70b",
            "llama3-1-405b-instruct-v1",
        ]
    ]
    """The ID of the model to use for chat completions.

    We only support the models listed here so far.
    """

    account_id: str
    """The account ID to use for usage tracking. This will be gradually enforced."""

    chat_template: str
    """Currently only supported for LLM-Engine models.

    A Jinja template string that defines how the chat completion API formats the
    string prompt. For Llama models, the template must take in at most a `messages`
    object, `bos_token` string, and `eos_token` string. The `messages` object is a
    list of dictionaries, each with keys `role` and `content`. For Mixtral models,
    the template must take in at most a `messages` object and `eos_token` string.
    The `messages` object looks identical to the Llama model's `messages` object,
    but the template can assume the `role` key takes on the values `user` or
    `assistant`, or `system` for the first message. The chat template either needs
    to handle this system message (which gets set via the `instructions` field or by
    the messages), or the `instructions` field must be set to `null` and the
    `messages` object must not contain any system messages.See the default chat
    template present in the Llama and Mixtral tokenizers for examples.
    """

    instructions: str
    """The initial instructions to provide to the chat completion model.

    Use this to guide the model to act in more specific ways. For example, if you
    have specific rules you want to restrict the model to follow you can specify
    them here.

    Good prompt engineering is crucial to getting performant results from the model.
    If you are having trouble getting the model to perform well, try writing more
    specific instructions here before trying more expensive techniques such as
    swapping in other models or finetuning the underlying LLM.
    """

    memory_strategy: MemoryStrategy
    """The memory strategy to use for the agent.

    A memory strategy is a way to prevent the underlying LLM's context limit from
    being exceeded. Each memory strategy uses a different technique to condense the
    input message list into a smaller payload for the underlying LLM.

    We only support the Last K memory strategy right now, but will be adding new
    strategies soon.
    """

    model_parameters: ModelParameters
    """
    Configuration parameters for the chat completion model, such as temperature,
    max_tokens, and stop_sequences.

    If not specified, the default value are:

    - temperature: 0.2
    - max_tokens: None (limited by the model's max tokens)
    - stop_sequences: None
    """


class MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsTextUserMessageContentParts(
    TypedDict, total=False
):
    text: Required[str]

    type: Literal["text"]


class MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentPartsImageURL(
    TypedDict, total=False
):
    url: Required[str]
    """The URL of the image. Note: only OpenAI supports this."""

    detail: Literal["low", "high", "auto"]
    """Only used for OpenAI. Corresponds to OpenAI's image detail parameter."""


class MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentParts(
    TypedDict, total=False
):
    image_url: Required[
        MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentPartsImageURL
    ]
    """Specifies the image URL and level of detail. Only supported by OpenAI models"""

    type: Literal["image_url"]


class MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentPartsImageData(
    TypedDict, total=False
):
    data: Required[str]
    """The base64-encoded image data."""

    media_type: Required[str]
    """The media/mime type of the image data.

    For example, 'image/png'. Check providers' documentation for supported media
    types.
    """

    detail: Literal["low", "high", "auto"]
    """Only used for OpenAI. Corresponds to OpenAI's image detail parameter."""

    type: Literal["base64"]
    """The type of the image data. Only base64 is supported."""


class MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentParts(
    TypedDict, total=False
):
    image_data: Required[
        MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentPartsImageData
    ]
    """Specifies inline image data"""

    type: Literal["image_data"]


MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1: TypeAlias = Union[
    MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsTextUserMessageContentParts,
    MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentParts,
    MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentParts,
]


class MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessage(TypedDict, total=False):
    content: Required[Union[str, Iterable[MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1]]]
    """Input from the user.

    Can either be text or a list of content parts. Not all models support image
    content parts, or multiple parts.
    """

    role: Literal["user"]
    """The role of the message. Must be set to 'user'.

    A user message is a message from the user to the AI. This should be the message
    used to send end user input to the AI.
    """


class MessageEgpAPIBackendServerAPIModelsEgpModelsAssistantMessage(TypedDict, total=False):
    content: Required[str]
    """Text response from the assistant"""

    role: Literal["assistant"]
    """The role of the message. Must be set to 'assistant'.

    An assistant message is a message from the AI to the client. It is different
    from an agent message in that it cannot contain a tool request. It is simply a
    direct response from the AI to the client.
    """


class MessageEgpAPIBackendServerAPIModelsEgpModelsSystemMessage(TypedDict, total=False):
    content: Required[str]
    """Text input from the system."""

    role: Literal["system"]
    """The role of the message. Must be set to 'system'.

    A system message is different from other messages in that it does not originate
    from a party engaged in a user/AI conversation. Instead, it is a message that is
    injected by either the application or system to guide the conversation. For
    example, a system message may be used as initial instructions for an AI entity
    or to tell the AI that it did not do something correctly.
    """


Message: TypeAlias = Union[
    MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessage,
    MessageEgpAPIBackendServerAPIModelsEgpModelsAssistantMessage,
    MessageEgpAPIBackendServerAPIModelsEgpModelsSystemMessage,
]


class MemoryStrategyParams(TypedDict, total=False):
    k: Required[int]
    """The maximum number of previous messages to remember."""


class MemoryStrategy(TypedDict, total=False):
    params: Required[MemoryStrategyParams]
    """Configuration parameters for the memory strategy."""

    name: Literal["last_k"]
    """Name of the memory strategy. Must be `last_k`.

    This strategy truncates the message history to the last `k` messages. It is the
    simplest way to prevent the model's context limit from being exceeded. However,
    this strategy only allows the model to have short term memory. For longer term
    memory, please use one of the other strategies.
    """


class ModelParameters(TypedDict, total=False):
    frequency_penalty: float
    """Penalize tokens based on how much they have already appeared in the text.

    Positive values encourage the model to generate new tokens and negative values
    encourage the model to repeat tokens. Available for models from LLM Engine, and
    OpenAI
    """

    max_tokens: int
    """The maximum number of tokens to generate in the completion.

    The token count of your prompt plus max_tokens cannot exceed the model's context
    length. If not, specified, max_tokens will be determined based on the model
    used: | Model API family | Model API default | EGP applied default | | --- | ---
    | --- | | OpenAI Completions |
    [`16`](https://platform.openai.com/docs/api-reference/completions/create#max_tokens)
    | `context window - prompt size` | | OpenAI Chat Completions |
    [`context window - prompt size`](https://platform.openai.com/docs/api-reference/chat/create#max_tokens)
    | `context window - prompt size` | | LLM Engine |
    [`max_new_tokens`](https://github.com/scaleapi/launch-python-client/blob/207adced1c88c1c2907266fa9dd1f1ff3ec0ea5b/launch/client.py#L2910)
    parameter is required | `100` | | Anthropic Claude 2 |
    [`max_tokens_to_sample`](https://docs.anthropic.com/claude/reference/complete_post)
    parameter is required | `10000` |
    """

    presence_penalty: float
    """Penalize tokens based on if they have already appeared in the text.

    Positive values encourage the model to generate new tokens and negative values
    encourage the model to repeat tokens. Available for models from LLM Engine, and
    OpenAI.
    """

    stop_sequences: List[str]
    """List of up to 4 sequences where the API will stop generating further tokens.

    The returned text will not contain the stop sequence.
    """

    temperature: float
    """What sampling temperature to use, between [0, 1].

    Higher values like 0.8 will make the output more random, while lower values like
    0.2 will make it more focused and deterministic. Setting temperature=0.0 will
    enable fully deterministic (greedy) sampling.
    """

    top_k: Annotated[int, PropertyInfo(alias="topK")]
    """Sample from the k most likely next tokens at each step.

    Lower k focuses on higher probability tokens. Available for models from
    Anthropic, Google, and LLM Engine.
    """

    top_p: Annotated[float, PropertyInfo(alias="topP")]
    """The cumulative probability cutoff for token selection.

    Lower values mean sampling from a smaller, more top-weighted nucleus. Available
    for models from Anthropic, Google, Mistral, LLM Engine, and OpenAI.
    """


class ChatCompletionCreateParamsNonStreaming(ChatCompletionCreateParamsBase, total=False):
    stream: Literal[False]
    """Whether or not to stream the response.

    Setting this to True will stream the completion in real-time.
    """


class ChatCompletionCreateParamsStreaming(ChatCompletionCreateParamsBase):
    stream: Required[Literal[True]]
    """Whether or not to stream the response.

    Setting this to True will stream the completion in real-time.
    """


ChatCompletionCreateParams = Union[ChatCompletionCreateParamsNonStreaming, ChatCompletionCreateParamsStreaming]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["GoogleDriveDataSourceConfig"]


class GoogleDriveDataSourceConfig(BaseModel):
    drive_id: str
    """ID associated with the Google Drive to retrieve contents from"""

    source: Literal["GoogleDrive"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

__all__ = ["TimeseryRetrieveParams"]


class TimeseryRetrieveParams(TypedDict, total=False):
    application_spec_id: Required[str]

    account_id: str
    """Account ID used for authorization"""

    from_ts: int
    """The starting (oldest) timestamp window in seconds."""

    to_ts: int
    """The ending (most recent) timestamp in seconds."""

    variants: List[str]
    """Which variants to filter on"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ConfluenceDataSourceAuthConfigParam"]


class ConfluenceDataSourceAuthConfigParam(TypedDict, total=False):
    api_key: Required[str]
    """
    API key to use for Confluence, set this to override api key configured in env
    vars.
    """

    atlassian_domain: Required[str]
    """
    Your Confluence API server's full domain, set to override domain configured in
    env vars. E.g. 'https://[your-company].atlassian.net'
    """

    client_email: Required[str]
    """
    Client email to use for Confluence, set to override client email set in env
    vars.
    """

    source: Required[Literal["Confluence"]]

    encrypted: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["SharePointDataSourceConfig"]


class SharePointDataSourceConfig(BaseModel):
    client_id: str
    """Client ID associated with this SharePoint site"""

    site_id: str
    """
    Site ID for this SharePoint site, can be found at
    https://[hostname].sharepoint.com/sites/[site name]/\\__api/site/id
    """

    source: Literal["SharePoint"]

    tenant_id: str
    """Tenant ID that the SharePoint site is within"""

    folder_path: Optional[str] = None
    """Nested folder path to read files from the root of the site.

    Please omit the leading slash. Example: 'Documents/sub_directory'
    """

    recursive: Optional[bool] = None
    """Recurse through the folder contents, default is True."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["EvaluationDatasetVersion"]


class EvaluationDatasetVersion(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    evaluation_dataset_id: str
    """The ID of the associated evaluation dataset."""

    num: int
    """The version number, automatically incremented on creation"""

    archived_at: Optional[datetime] = None
    """The date and time when the entity was archived in ISO format."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._compat import PYDANTIC_V2, ConfigDict
from .._models import BaseModel

__all__ = ["ModelInstance", "BaseModelMetadata", "BaseModelMetadataModelDetails"]


class BaseModelMetadataModelDetails(BaseModel):
    alignments: Optional[int] = None

    languages: Optional[int] = None

    number_of_parameters: Optional[int] = None

    token_context_window: Optional[int] = None


class BaseModelMetadata(BaseModel):
    delivery_date: Optional[str] = None

    model_developer: Optional[str] = None

    model_license_url: Optional[str] = None

    model_details: Optional[BaseModelMetadataModelDetails] = FieldInfo(alias="modelDetails", default=None)

    ui_model_section_type: Optional[Literal["PARTNER", "OPENSOURCE", "CUSTOM"]] = None
    """An enumeration."""

    if PYDANTIC_V2:
        # allow fields with a `model_` prefix
        model_config = ConfigDict(protected_namespaces=tuple())


class ModelInstance(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    deployment_count: int
    """Number of deployments of this model instance"""

    model_type: Literal["COMPLETION", "CHAT_COMPLETION", "AGENT", "EMBEDDING", "RERANKING", "GENERIC"]
    """An enum representing the different types of models supported.

    Attributes: COMPLETION: Denotes that the model type is completion.
    CHAT_COMPLETION: Denotes that the model type is chat completion. AGENT: Denotes
    that the model type is agent. EMBEDDING: Denotes that the model type is
    embedding. RERANKING: Denotes that the model type is reranking. GENERIC: Denotes
    that the model type is generic.
    """

    name: str

    request_schema: object
    """JSON schema for the requests to the model instance"""

    response_schema: object
    """JSON schema for the response to the model"""

    supports_multi_turn: bool
    """True if a model supports multi-turn conversations natively"""

    base_model_id: Optional[str] = None

    base_model_metadata: Optional[BaseModelMetadata] = None

    description: Optional[str] = None

    model_card: Optional[str] = None

    model_creation_parameters: Optional[object] = None

    model_group_id: Optional[str] = None

    model_template_id: Optional[str] = None

    model_vendor: Optional[
        Literal["OPENAI", "COHERE", "GOOGLE", "ANTHROPIC", "LAUNCH", "LLMENGINE", "BEDROCK", "OTHER"]
    ] = None
    """An enum representing the different types of model vendors supported.

    Attributes: OPENAI: Denotes that the model vendor is OpenAI. COHERE: Denotes
    that the model vendor is Cohere. GOOGLE: Denotes that the model vendor is
    Google. ANTHROPIC: Denotes that the model vendor is Anthropic. LLMENGINE:
    Denotes that the model vendor is LLM Engine. OTHER: Denotes that the model
    vendor is Other.
    """

    training_data_card: Optional[str] = None

    if PYDANTIC_V2:
        # allow fields with a `model_` prefix
        model_config = ConfigDict(protected_namespaces=tuple())



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["MessageUpdateParams"]


class MessageUpdateParams(TypedDict, total=False):
    thread_id: Required[str]

    output_text: str
    """The output text generated by the application."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ContributorMetricListParams"]


class ContributorMetricListParams(TypedDict, total=False):
    limit: int
    """Maximum number of artifacts to be returned by the given endpoint.

    Defaults to 100 and cannot be greater than 10k.
    """

    page: int
    """Page number for pagination to be returned by the given endpoint.

    Starts at page 1
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

__all__ = ["TestCaseResultRetrieveParams"]


class TestCaseResultRetrieveParams(TypedDict, total=False):
    evaluation_id: Required[str]

    view: List[Literal["AnnotationResults", "Metrics", "Task", "TestCaseVersion"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from ..._models import BaseModel

__all__ = ["CreateKnowledgeBaseUploadResponse"]


class CreateKnowledgeBaseUploadResponse(BaseModel):
    upload_id: str
    """ID of the created knowledge base upload job."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, TypeAlias, TypedDict

from .chunk_extra_info_schema import ChunkExtraInfoSchema
from .string_extra_info_schema import StringExtraInfoSchema

__all__ = ["ResultSchemaGeneration", "GenerationExtraInfo"]

GenerationExtraInfo: TypeAlias = Union[ChunkExtraInfoSchema, StringExtraInfoSchema]


class ResultSchemaGeneration(TypedDict, total=False):
    generation_output: Required[str]

    generation_extra_info: GenerationExtraInfo



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["RerankingResponse"]


class RerankingResponse(BaseModel):
    chunk_scores: List[float]

    tokens_used: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.types import (
    KnowledgeBase,
    CreateKnowledgeBaseResponse,
    DeleteKnowledgeBaseResponse,
    CreateKnowledgeBaseUploadsFromFilesResponse,
)
from scale_gp.pagination import SyncPageResponse, AsyncPageResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestKnowledgeBases:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        knowledge_base = client.knowledge_bases.create(
            embedding_config={
                "model_deployment_id": "model_deployment_id",
                "type": "models_api",
            },
            knowledge_base_name="knowledge_base_name",
        )
        assert_matches_type(CreateKnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        knowledge_base = client.knowledge_bases.create(
            embedding_config={
                "model_deployment_id": "model_deployment_id",
                "type": "models_api",
            },
            knowledge_base_name="knowledge_base_name",
            account_id="account_id",
            metadata={},
        )
        assert_matches_type(CreateKnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.knowledge_bases.with_raw_response.create(
            embedding_config={
                "model_deployment_id": "model_deployment_id",
                "type": "models_api",
            },
            knowledge_base_name="knowledge_base_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(CreateKnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.knowledge_bases.with_streaming_response.create(
            embedding_config={
                "model_deployment_id": "model_deployment_id",
                "type": "models_api",
            },
            knowledge_base_name="knowledge_base_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(CreateKnowledgeBaseResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        knowledge_base = client.knowledge_bases.retrieve(
            knowledge_base_id="knowledge_base_id",
        )
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params(self, client: SGPClient) -> None:
        knowledge_base = client.knowledge_bases.retrieve(
            knowledge_base_id="knowledge_base_id",
            include_artifacts_status=True,
            view=["Connections", "ArtifactCount"],
        )
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.knowledge_bases.with_raw_response.retrieve(
            knowledge_base_id="knowledge_base_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.knowledge_bases.with_streaming_response.retrieve(
            knowledge_base_id="knowledge_base_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            client.knowledge_bases.with_raw_response.retrieve(
                knowledge_base_id="",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        knowledge_base = client.knowledge_bases.list()
        assert_matches_type(SyncPageResponse[KnowledgeBase], knowledge_base, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        knowledge_base = client.knowledge_bases.list(
            account_id="account_id",
            limit=1,
            page=1,
            view=["Connections", "ArtifactCount"],
        )
        assert_matches_type(SyncPageResponse[KnowledgeBase], knowledge_base, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.knowledge_bases.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(SyncPageResponse[KnowledgeBase], knowledge_base, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.knowledge_bases.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(SyncPageResponse[KnowledgeBase], knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: SGPClient) -> None:
        knowledge_base = client.knowledge_bases.delete(
            "knowledge_base_id",
        )
        assert_matches_type(DeleteKnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: SGPClient) -> None:
        response = client.knowledge_bases.with_raw_response.delete(
            "knowledge_base_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(DeleteKnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: SGPClient) -> None:
        with client.knowledge_bases.with_streaming_response.delete(
            "knowledge_base_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(DeleteKnowledgeBaseResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            client.knowledge_bases.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_query(self, client: SGPClient) -> None:
        knowledge_base = client.knowledge_bases.query(
            knowledge_base_id="knowledge_base_id",
            query="query",
            top_k=1,
        )
        assert_matches_type(object, knowledge_base, path=["response"])

    @parametrize
    def test_method_query_with_all_params(self, client: SGPClient) -> None:
        knowledge_base = client.knowledge_bases.query(
            knowledge_base_id="knowledge_base_id",
            query="query",
            top_k=1,
            include_embeddings=True,
            metadata_filters={},
            verbose=True,
            wildcard_filters={
                "foo": {
                    "value": "value",
                    "case_insensitive": True,
                }
            },
        )
        assert_matches_type(object, knowledge_base, path=["response"])

    @parametrize
    def test_raw_response_query(self, client: SGPClient) -> None:
        response = client.knowledge_bases.with_raw_response.query(
            knowledge_base_id="knowledge_base_id",
            query="query",
            top_k=1,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(object, knowledge_base, path=["response"])

    @parametrize
    def test_streaming_response_query(self, client: SGPClient) -> None:
        with client.knowledge_bases.with_streaming_response.query(
            knowledge_base_id="knowledge_base_id",
            query="query",
            top_k=1,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(object, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_query(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            client.knowledge_bases.with_raw_response.query(
                knowledge_base_id="",
                query="query",
                top_k=1,
            )

    @pytest.mark.skip(reason="prism_binary_unsupported")
    @parametrize
    def test_method_upload_files(self, client: SGPClient) -> None:
        knowledge_base = client.knowledge_bases.upload_files(
            knowledge_base_id="knowledge_base_id",
            chunking_strategy_config="chunking_strategy_config",
            data_source_config="data_source_config",
            files=[b"raw file contents", b"raw file contents", b"raw file contents"],
            force_reupload=True,
        )
        assert_matches_type(CreateKnowledgeBaseUploadsFromFilesResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="prism_binary_unsupported")
    @parametrize
    def test_raw_response_upload_files(self, client: SGPClient) -> None:
        response = client.knowledge_bases.with_raw_response.upload_files(
            knowledge_base_id="knowledge_base_id",
            chunking_strategy_config="chunking_strategy_config",
            data_source_config="data_source_config",
            files=[b"raw file contents", b"raw file contents", b"raw file contents"],
            force_reupload=True,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(CreateKnowledgeBaseUploadsFromFilesResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="prism_binary_unsupported")
    @parametrize
    def test_streaming_response_upload_files(self, client: SGPClient) -> None:
        with client.knowledge_bases.with_streaming_response.upload_files(
            knowledge_base_id="knowledge_base_id",
            chunking_strategy_config="chunking_strategy_config",
            data_source_config="data_source_config",
            files=[b"raw file contents", b"raw file contents", b"raw file contents"],
            force_reupload=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(CreateKnowledgeBaseUploadsFromFilesResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="prism_binary_unsupported")
    @parametrize
    def test_path_params_upload_files(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            client.knowledge_bases.with_raw_response.upload_files(
                knowledge_base_id="",
                chunking_strategy_config="chunking_strategy_config",
                data_source_config="data_source_config",
                files=[b"raw file contents", b"raw file contents", b"raw file contents"],
                force_reupload=True,
            )


class TestAsyncKnowledgeBases:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        knowledge_base = await async_client.knowledge_bases.create(
            embedding_config={
                "model_deployment_id": "model_deployment_id",
                "type": "models_api",
            },
            knowledge_base_name="knowledge_base_name",
        )
        assert_matches_type(CreateKnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        knowledge_base = await async_client.knowledge_bases.create(
            embedding_config={
                "model_deployment_id": "model_deployment_id",
                "type": "models_api",
            },
            knowledge_base_name="knowledge_base_name",
            account_id="account_id",
            metadata={},
        )
        assert_matches_type(CreateKnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_bases.with_raw_response.create(
            embedding_config={
                "model_deployment_id": "model_deployment_id",
                "type": "models_api",
            },
            knowledge_base_name="knowledge_base_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(CreateKnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_bases.with_streaming_response.create(
            embedding_config={
                "model_deployment_id": "model_deployment_id",
                "type": "models_api",
            },
            knowledge_base_name="knowledge_base_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(CreateKnowledgeBaseResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        knowledge_base = await async_client.knowledge_bases.retrieve(
            knowledge_base_id="knowledge_base_id",
        )
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncSGPClient) -> None:
        knowledge_base = await async_client.knowledge_bases.retrieve(
            knowledge_base_id="knowledge_base_id",
            include_artifacts_status=True,
            view=["Connections", "ArtifactCount"],
        )
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_bases.with_raw_response.retrieve(
            knowledge_base_id="knowledge_base_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_bases.with_streaming_response.retrieve(
            knowledge_base_id="knowledge_base_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            await async_client.knowledge_bases.with_raw_response.retrieve(
                knowledge_base_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        knowledge_base = await async_client.knowledge_bases.list()
        assert_matches_type(AsyncPageResponse[KnowledgeBase], knowledge_base, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        knowledge_base = await async_client.knowledge_bases.list(
            account_id="account_id",
            limit=1,
            page=1,
            view=["Connections", "ArtifactCount"],
        )
        assert_matches_type(AsyncPageResponse[KnowledgeBase], knowledge_base, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_bases.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(AsyncPageResponse[KnowledgeBase], knowledge_base, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_bases.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(AsyncPageResponse[KnowledgeBase], knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncSGPClient) -> None:
        knowledge_base = await async_client.knowledge_bases.delete(
            "knowledge_base_id",
        )
        assert_matches_type(DeleteKnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_bases.with_raw_response.delete(
            "knowledge_base_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(DeleteKnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_bases.with_streaming_response.delete(
            "knowledge_base_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(DeleteKnowledgeBaseResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            await async_client.knowledge_bases.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_query(self, async_client: AsyncSGPClient) -> None:
        knowledge_base = await async_client.knowledge_bases.query(
            knowledge_base_id="knowledge_base_id",
            query="query",
            top_k=1,
        )
        assert_matches_type(object, knowledge_base, path=["response"])

    @parametrize
    async def test_method_query_with_all_params(self, async_client: AsyncSGPClient) -> None:
        knowledge_base = await async_client.knowledge_bases.query(
            knowledge_base_id="knowledge_base_id",
            query="query",
            top_k=1,
            include_embeddings=True,
            metadata_filters={},
            verbose=True,
            wildcard_filters={
                "foo": {
                    "value": "value",
                    "case_insensitive": True,
                }
            },
        )
        assert_matches_type(object, knowledge_base, path=["response"])

    @parametrize
    async def test_raw_response_query(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_bases.with_raw_response.query(
            knowledge_base_id="knowledge_base_id",
            query="query",
            top_k=1,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(object, knowledge_base, path=["response"])

    @parametrize
    async def test_streaming_response_query(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_bases.with_streaming_response.query(
            knowledge_base_id="knowledge_base_id",
            query="query",
            top_k=1,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(object, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_query(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            await async_client.knowledge_bases.with_raw_response.query(
                knowledge_base_id="",
                query="query",
                top_k=1,
            )

    @pytest.mark.skip(reason="prism_binary_unsupported")
    @parametrize
    async def test_method_upload_files(self, async_client: AsyncSGPClient) -> None:
        knowledge_base = await async_client.knowledge_bases.upload_files(
            knowledge_base_id="knowledge_base_id",
            chunking_strategy_config="chunking_strategy_config",
            data_source_config="data_source_config",
            files=[b"raw file contents", b"raw file contents", b"raw file contents"],
            force_reupload=True,
        )
        assert_matches_type(CreateKnowledgeBaseUploadsFromFilesResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="prism_binary_unsupported")
    @parametrize
    async def test_raw_response_upload_files(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_bases.with_raw_response.upload_files(
            knowledge_base_id="knowledge_base_id",
            chunking_strategy_config="chunking_strategy_config",
            data_source_config="data_source_config",
            files=[b"raw file contents", b"raw file contents", b"raw file contents"],
            force_reupload=True,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(CreateKnowledgeBaseUploadsFromFilesResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="prism_binary_unsupported")
    @parametrize
    async def test_streaming_response_upload_files(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_bases.with_streaming_response.upload_files(
            knowledge_base_id="knowledge_base_id",
            chunking_strategy_config="chunking_strategy_config",
            data_source_config="data_source_config",
            files=[b"raw file contents", b"raw file contents", b"raw file contents"],
            force_reupload=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(CreateKnowledgeBaseUploadsFromFilesResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="prism_binary_unsupported")
    @parametrize
    async def test_path_params_upload_files(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            await async_client.knowledge_bases.with_raw_response.upload_files(
                knowledge_base_id="",
                chunking_strategy_config="chunking_strategy_config",
                data_source_config="data_source_config",
                files=[b"raw file contents", b"raw file contents", b"raw file contents"],
                force_reupload=True,
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .metrics import (
    MetricsResource,
    AsyncMetricsResource,
    MetricsResourceWithRawResponse,
    AsyncMetricsResourceWithRawResponse,
    MetricsResourceWithStreamingResponse,
    AsyncMetricsResourceWithStreamingResponse,
)
from .dashboards import (
    DashboardsResource,
    AsyncDashboardsResource,
    DashboardsResourceWithRawResponse,
    AsyncDashboardsResourceWithRawResponse,
    DashboardsResourceWithStreamingResponse,
    AsyncDashboardsResourceWithStreamingResponse,
)
from .applications import (
    ApplicationsResource,
    AsyncApplicationsResource,
    ApplicationsResourceWithRawResponse,
    AsyncApplicationsResourceWithRawResponse,
    ApplicationsResourceWithStreamingResponse,
    AsyncApplicationsResourceWithStreamingResponse,
)
from .chat_threads import (
    ChatThreadsResource,
    AsyncChatThreadsResource,
    ChatThreadsResourceWithRawResponse,
    AsyncChatThreadsResourceWithRawResponse,
    ChatThreadsResourceWithStreamingResponse,
    AsyncChatThreadsResourceWithStreamingResponse,
)

__all__ = [
    "ChatThreadsResource",
    "AsyncChatThreadsResource",
    "ChatThreadsResourceWithRawResponse",
    "AsyncChatThreadsResourceWithRawResponse",
    "ChatThreadsResourceWithStreamingResponse",
    "AsyncChatThreadsResourceWithStreamingResponse",
    "DashboardsResource",
    "AsyncDashboardsResource",
    "DashboardsResourceWithRawResponse",
    "AsyncDashboardsResourceWithRawResponse",
    "DashboardsResourceWithStreamingResponse",
    "AsyncDashboardsResourceWithStreamingResponse",
    "MetricsResource",
    "AsyncMetricsResource",
    "MetricsResourceWithRawResponse",
    "AsyncMetricsResourceWithRawResponse",
    "MetricsResourceWithStreamingResponse",
    "AsyncMetricsResourceWithStreamingResponse",
    "ApplicationsResource",
    "AsyncApplicationsResource",
    "ApplicationsResourceWithRawResponse",
    "AsyncApplicationsResourceWithRawResponse",
    "ApplicationsResourceWithStreamingResponse",
    "AsyncApplicationsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncPageResponse, AsyncPageResponse
from ..._base_client import AsyncPaginator, make_request_options
from ...types.knowledge_bases import (
    upload_schedule_list_params,
    upload_schedule_create_params,
    upload_schedule_update_params,
    upload_schedule_retrieve_params,
)
from ...types.shared.generic_delete_response import GenericDeleteResponse
from ...types.knowledge_bases.upload_schedule import UploadSchedule
from ...types.knowledge_bases.upload_schedule_with_views import UploadScheduleWithViews

__all__ = ["UploadSchedulesResource", "AsyncUploadSchedulesResource"]


class UploadSchedulesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UploadSchedulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return UploadSchedulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UploadSchedulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return UploadSchedulesResourceWithStreamingResponse(self)

    def create(
        self,
        knowledge_base_id: str,
        *,
        chunking_strategy_config: upload_schedule_create_params.ChunkingStrategyConfig,
        interval: float,
        knowledge_base_data_source_id: str,
        account_id: str | NotGiven = NOT_GIVEN,
        next_run_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UploadSchedule:
        """
        ### Description

        Creates a upload schedule

        ### Details

        This API can be used to create a upload schedule. To use this API, review the
        request schema and pass in all fields that are required to create a upload
        schedule.

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._post(
            f"/v4/knowledge-bases/{knowledge_base_id}/upload-schedules",
            body=maybe_transform(
                {
                    "chunking_strategy_config": chunking_strategy_config,
                    "interval": interval,
                    "knowledge_base_data_source_id": knowledge_base_data_source_id,
                    "account_id": account_id,
                    "next_run_at": next_run_at,
                },
                upload_schedule_create_params.UploadScheduleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UploadSchedule,
        )

    def retrieve(
        self,
        upload_schedule_id: str,
        *,
        knowledge_base_id: str,
        view: List[Literal["DataSource"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UploadScheduleWithViews:
        """
        ### Description

        Gets the details of a upload schedule

        ### Details

        This API can be used to get information about a single upload schedule by ID. To
        use this API, pass in the `id` that was returned from your Create Upload
        Schedule API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not upload_schedule_id:
            raise ValueError(f"Expected a non-empty value for `upload_schedule_id` but received {upload_schedule_id!r}")
        return self._get(
            f"/v4/knowledge-bases/{knowledge_base_id}/upload-schedules/{upload_schedule_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"view": view}, upload_schedule_retrieve_params.UploadScheduleRetrieveParams),
            ),
            cast_to=UploadScheduleWithViews,
        )

    def update(
        self,
        upload_schedule_id: str,
        *,
        knowledge_base_id: str,
        chunking_strategy_config: upload_schedule_update_params.ChunkingStrategyConfig | NotGiven = NOT_GIVEN,
        interval: float | NotGiven = NOT_GIVEN,
        next_run_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UploadSchedule:
        """
        ### Description

        Updates a upload schedule

        ### Details

        This API can be used to update the upload schedule that matches the ID that was
        passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Upload Schedule API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not upload_schedule_id:
            raise ValueError(f"Expected a non-empty value for `upload_schedule_id` but received {upload_schedule_id!r}")
        return self._patch(
            f"/v4/knowledge-bases/{knowledge_base_id}/upload-schedules/{upload_schedule_id}",
            body=maybe_transform(
                {
                    "chunking_strategy_config": chunking_strategy_config,
                    "interval": interval,
                    "next_run_at": next_run_at,
                },
                upload_schedule_update_params.UploadScheduleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UploadSchedule,
        )

    def list(
        self,
        knowledge_base_id: str,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        view: List[Literal["DataSource"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[UploadScheduleWithViews]:
        """
        ### Description

        Lists all upload schedules accessible to the user.

        ### Details

        This API can be used to list upload schedules. If a user has access to multiple
        accounts, all upload schedules from all accounts the user is associated with
        will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._get_api_list(
            f"/v4/knowledge-bases/{knowledge_base_id}/upload-schedules",
            page=SyncPageResponse[UploadScheduleWithViews],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                        "view": view,
                    },
                    upload_schedule_list_params.UploadScheduleListParams,
                ),
            ),
            model=UploadScheduleWithViews,
        )

    def delete(
        self,
        upload_schedule_id: str,
        *,
        knowledge_base_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a upload schedule

        ### Details

        This API can be used to delete a upload schedule by ID. To use this API, pass in
        the `id` that was returned from your Create Upload Schedule API call as a path
        parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not upload_schedule_id:
            raise ValueError(f"Expected a non-empty value for `upload_schedule_id` but received {upload_schedule_id!r}")
        return self._delete(
            f"/v4/knowledge-bases/{knowledge_base_id}/upload-schedules/{upload_schedule_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class AsyncUploadSchedulesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUploadSchedulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUploadSchedulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUploadSchedulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncUploadSchedulesResourceWithStreamingResponse(self)

    async def create(
        self,
        knowledge_base_id: str,
        *,
        chunking_strategy_config: upload_schedule_create_params.ChunkingStrategyConfig,
        interval: float,
        knowledge_base_data_source_id: str,
        account_id: str | NotGiven = NOT_GIVEN,
        next_run_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UploadSchedule:
        """
        ### Description

        Creates a upload schedule

        ### Details

        This API can be used to create a upload schedule. To use this API, review the
        request schema and pass in all fields that are required to create a upload
        schedule.

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return await self._post(
            f"/v4/knowledge-bases/{knowledge_base_id}/upload-schedules",
            body=await async_maybe_transform(
                {
                    "chunking_strategy_config": chunking_strategy_config,
                    "interval": interval,
                    "knowledge_base_data_source_id": knowledge_base_data_source_id,
                    "account_id": account_id,
                    "next_run_at": next_run_at,
                },
                upload_schedule_create_params.UploadScheduleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UploadSchedule,
        )

    async def retrieve(
        self,
        upload_schedule_id: str,
        *,
        knowledge_base_id: str,
        view: List[Literal["DataSource"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UploadScheduleWithViews:
        """
        ### Description

        Gets the details of a upload schedule

        ### Details

        This API can be used to get information about a single upload schedule by ID. To
        use this API, pass in the `id` that was returned from your Create Upload
        Schedule API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not upload_schedule_id:
            raise ValueError(f"Expected a non-empty value for `upload_schedule_id` but received {upload_schedule_id!r}")
        return await self._get(
            f"/v4/knowledge-bases/{knowledge_base_id}/upload-schedules/{upload_schedule_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"view": view}, upload_schedule_retrieve_params.UploadScheduleRetrieveParams
                ),
            ),
            cast_to=UploadScheduleWithViews,
        )

    async def update(
        self,
        upload_schedule_id: str,
        *,
        knowledge_base_id: str,
        chunking_strategy_config: upload_schedule_update_params.ChunkingStrategyConfig | NotGiven = NOT_GIVEN,
        interval: float | NotGiven = NOT_GIVEN,
        next_run_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UploadSchedule:
        """
        ### Description

        Updates a upload schedule

        ### Details

        This API can be used to update the upload schedule that matches the ID that was
        passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Upload Schedule API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not upload_schedule_id:
            raise ValueError(f"Expected a non-empty value for `upload_schedule_id` but received {upload_schedule_id!r}")
        return await self._patch(
            f"/v4/knowledge-bases/{knowledge_base_id}/upload-schedules/{upload_schedule_id}",
            body=await async_maybe_transform(
                {
                    "chunking_strategy_config": chunking_strategy_config,
                    "interval": interval,
                    "next_run_at": next_run_at,
                },
                upload_schedule_update_params.UploadScheduleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UploadSchedule,
        )

    def list(
        self,
        knowledge_base_id: str,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        view: List[Literal["DataSource"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[UploadScheduleWithViews, AsyncPageResponse[UploadScheduleWithViews]]:
        """
        ### Description

        Lists all upload schedules accessible to the user.

        ### Details

        This API can be used to list upload schedules. If a user has access to multiple
        accounts, all upload schedules from all accounts the user is associated with
        will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._get_api_list(
            f"/v4/knowledge-bases/{knowledge_base_id}/upload-schedules",
            page=AsyncPageResponse[UploadScheduleWithViews],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                        "view": view,
                    },
                    upload_schedule_list_params.UploadScheduleListParams,
                ),
            ),
            model=UploadScheduleWithViews,
        )

    async def delete(
        self,
        upload_schedule_id: str,
        *,
        knowledge_base_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a upload schedule

        ### Details

        This API can be used to delete a upload schedule by ID. To use this API, pass in
        the `id` that was returned from your Create Upload Schedule API call as a path
        parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not upload_schedule_id:
            raise ValueError(f"Expected a non-empty value for `upload_schedule_id` but received {upload_schedule_id!r}")
        return await self._delete(
            f"/v4/knowledge-bases/{knowledge_base_id}/upload-schedules/{upload_schedule_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class UploadSchedulesResourceWithRawResponse:
    def __init__(self, upload_schedules: UploadSchedulesResource) -> None:
        self._upload_schedules = upload_schedules

        self.create = to_raw_response_wrapper(
            upload_schedules.create,
        )
        self.retrieve = to_raw_response_wrapper(
            upload_schedules.retrieve,
        )
        self.update = to_raw_response_wrapper(
            upload_schedules.update,
        )
        self.list = to_raw_response_wrapper(
            upload_schedules.list,
        )
        self.delete = to_raw_response_wrapper(
            upload_schedules.delete,
        )


class AsyncUploadSchedulesResourceWithRawResponse:
    def __init__(self, upload_schedules: AsyncUploadSchedulesResource) -> None:
        self._upload_schedules = upload_schedules

        self.create = async_to_raw_response_wrapper(
            upload_schedules.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            upload_schedules.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            upload_schedules.update,
        )
        self.list = async_to_raw_response_wrapper(
            upload_schedules.list,
        )
        self.delete = async_to_raw_response_wrapper(
            upload_schedules.delete,
        )


class UploadSchedulesResourceWithStreamingResponse:
    def __init__(self, upload_schedules: UploadSchedulesResource) -> None:
        self._upload_schedules = upload_schedules

        self.create = to_streamed_response_wrapper(
            upload_schedules.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            upload_schedules.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            upload_schedules.update,
        )
        self.list = to_streamed_response_wrapper(
            upload_schedules.list,
        )
        self.delete = to_streamed_response_wrapper(
            upload_schedules.delete,
        )


class AsyncUploadSchedulesResourceWithStreamingResponse:
    def __init__(self, upload_schedules: AsyncUploadSchedulesResource) -> None:
        self._upload_schedules = upload_schedules

        self.create = async_to_streamed_response_wrapper(
            upload_schedules.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            upload_schedules.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            upload_schedules.update,
        )
        self.list = async_to_streamed_response_wrapper(
            upload_schedules.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            upload_schedules.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncPageResponse, AsyncPageResponse
from ..._base_client import AsyncPaginator, make_request_options
from ...types.evaluations import contributor_metric_list_params
from ...types.evaluations.contributor_metrics import ContributorMetrics

__all__ = ["ContributorMetricsResource", "AsyncContributorMetricsResource"]


class ContributorMetricsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ContributorMetricsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ContributorMetricsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ContributorMetricsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ContributorMetricsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        contributor_id: str,
        *,
        evaluation_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ContributorMetrics:
        """
        Get Contributor Metrics

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not contributor_id:
            raise ValueError(f"Expected a non-empty value for `contributor_id` but received {contributor_id!r}")
        return self._get(
            f"/v4/evaluations/{evaluation_id}/contributor-metrics/{contributor_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContributorMetrics,
        )

    def list(
        self,
        evaluation_id: str,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[ContributorMetrics]:
        """
        List Contributor Metrics

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return self._get_api_list(
            f"/v4/evaluations/{evaluation_id}/contributor-metrics",
            page=SyncPageResponse[ContributorMetrics],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "page": page,
                    },
                    contributor_metric_list_params.ContributorMetricListParams,
                ),
            ),
            model=ContributorMetrics,
        )


class AsyncContributorMetricsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncContributorMetricsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncContributorMetricsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncContributorMetricsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncContributorMetricsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        contributor_id: str,
        *,
        evaluation_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ContributorMetrics:
        """
        Get Contributor Metrics

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not contributor_id:
            raise ValueError(f"Expected a non-empty value for `contributor_id` but received {contributor_id!r}")
        return await self._get(
            f"/v4/evaluations/{evaluation_id}/contributor-metrics/{contributor_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContributorMetrics,
        )

    def list(
        self,
        evaluation_id: str,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ContributorMetrics, AsyncPageResponse[ContributorMetrics]]:
        """
        List Contributor Metrics

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return self._get_api_list(
            f"/v4/evaluations/{evaluation_id}/contributor-metrics",
            page=AsyncPageResponse[ContributorMetrics],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "page": page,
                    },
                    contributor_metric_list_params.ContributorMetricListParams,
                ),
            ),
            model=ContributorMetrics,
        )


class ContributorMetricsResourceWithRawResponse:
    def __init__(self, contributor_metrics: ContributorMetricsResource) -> None:
        self._contributor_metrics = contributor_metrics

        self.retrieve = to_raw_response_wrapper(
            contributor_metrics.retrieve,
        )
        self.list = to_raw_response_wrapper(
            contributor_metrics.list,
        )


class AsyncContributorMetricsResourceWithRawResponse:
    def __init__(self, contributor_metrics: AsyncContributorMetricsResource) -> None:
        self._contributor_metrics = contributor_metrics

        self.retrieve = async_to_raw_response_wrapper(
            contributor_metrics.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            contributor_metrics.list,
        )


class ContributorMetricsResourceWithStreamingResponse:
    def __init__(self, contributor_metrics: ContributorMetricsResource) -> None:
        self._contributor_metrics = contributor_metrics

        self.retrieve = to_streamed_response_wrapper(
            contributor_metrics.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            contributor_metrics.list,
        )


class AsyncContributorMetricsResourceWithStreamingResponse:
    def __init__(self, contributor_metrics: AsyncContributorMetricsResource) -> None:
        self._contributor_metrics = contributor_metrics

        self.retrieve = async_to_streamed_response_wrapper(
            contributor_metrics.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            contributor_metrics.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from ..shared.chunk_extra_info_schema import ChunkExtraInfoSchema
from ..shared.string_extra_info_schema import StringExtraInfoSchema

__all__ = ["GenerationTestCaseSchema", "ExpectedExtraInfo"]

ExpectedExtraInfo: TypeAlias = Annotated[
    Union[ChunkExtraInfoSchema, StringExtraInfoSchema], PropertyInfo(discriminator="schema_type")
]


class GenerationTestCaseSchema(BaseModel):
    input: str

    expected_extra_info: Optional[ExpectedExtraInfo] = None

    expected_output: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, List, Union, Iterable, cast
from typing_extensions import Literal

import httpx

from ..types import (
    application_test_case_output_list_params,
    application_test_case_output_batch_params,
    application_test_case_output_retrieve_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncPageResponse, AsyncPageResponse
from .._base_client import AsyncPaginator, make_request_options
from ..types.application_test_case_output_list_response import ApplicationTestCaseOutputListResponse
from ..types.application_test_case_output_batch_response import ApplicationTestCaseOutputBatchResponse
from ..types.application_test_case_output_retrieve_response import ApplicationTestCaseOutputRetrieveResponse

__all__ = ["ApplicationTestCaseOutputsResource", "AsyncApplicationTestCaseOutputsResource"]


class ApplicationTestCaseOutputsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ApplicationTestCaseOutputsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ApplicationTestCaseOutputsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ApplicationTestCaseOutputsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ApplicationTestCaseOutputsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        application_test_case_output_id: str,
        *,
        view: List[Literal["MetricScores", "TestCaseVersion", "Trace"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationTestCaseOutputRetrieveResponse:
        """
        ### Description

        Gets the details of a application test case output

        ### Details

        This API can be used to get information about a single application test case
        output by ID. To use this API, pass in the `id` that was returned from your
        Create Application Test Case Output API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_test_case_output_id:
            raise ValueError(
                f"Expected a non-empty value for `application_test_case_output_id` but received {application_test_case_output_id!r}"
            )
        return cast(
            ApplicationTestCaseOutputRetrieveResponse,
            self._get(
                f"/v4/application-test-case-outputs/{application_test_case_output_id}",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=maybe_transform(
                        {"view": view},
                        application_test_case_output_retrieve_params.ApplicationTestCaseOutputRetrieveParams,
                    ),
                ),
                cast_to=cast(
                    Any, ApplicationTestCaseOutputRetrieveResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        application_variant_id: Union[int, str] | NotGiven = NOT_GIVEN,
        application_variant_report_id: Union[int, str] | NotGiven = NOT_GIVEN,
        evaluation_dataset_id: Union[int, str] | NotGiven = NOT_GIVEN,
        evaluation_dataset_version_num: Union[int, str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        view: List[Literal["MetricScores", "TestCaseVersion", "Trace"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[ApplicationTestCaseOutputListResponse]:
        """
        ### Description

        Lists all application test case outputs accessible to the user.

        ### Details

        This API can be used to list application test case outputs. If a user has access
        to multiple accounts, all application test case outputs from all accounts the
        user is associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/application-test-case-outputs",
            page=SyncPageResponse[ApplicationTestCaseOutputListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "application_variant_id": application_variant_id,
                        "application_variant_report_id": application_variant_report_id,
                        "evaluation_dataset_id": evaluation_dataset_id,
                        "evaluation_dataset_version_num": evaluation_dataset_version_num,
                        "limit": limit,
                        "page": page,
                        "view": view,
                    },
                    application_test_case_output_list_params.ApplicationTestCaseOutputListParams,
                ),
            ),
            model=cast(
                Any, ApplicationTestCaseOutputListResponse
            ),  # Union types cannot be passed in as arguments in the type system
        )

    def batch(
        self,
        *,
        items: Iterable[application_test_case_output_batch_params.Item],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationTestCaseOutputBatchResponse:
        """
        ### Description

        Creates a batch of application test case outputs

        ### Details

        This API can be used to create multiple application test case outputs so users
        do not have to the incur the cost of repeated network calls. To use this API,
        pass in a list of application test case outputs in the request body.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/application-test-case-outputs/batch",
            body=maybe_transform(items, Iterable[application_test_case_output_batch_params.Item]),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationTestCaseOutputBatchResponse,
        )


class AsyncApplicationTestCaseOutputsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncApplicationTestCaseOutputsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncApplicationTestCaseOutputsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncApplicationTestCaseOutputsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncApplicationTestCaseOutputsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        application_test_case_output_id: str,
        *,
        view: List[Literal["MetricScores", "TestCaseVersion", "Trace"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationTestCaseOutputRetrieveResponse:
        """
        ### Description

        Gets the details of a application test case output

        ### Details

        This API can be used to get information about a single application test case
        output by ID. To use this API, pass in the `id` that was returned from your
        Create Application Test Case Output API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_test_case_output_id:
            raise ValueError(
                f"Expected a non-empty value for `application_test_case_output_id` but received {application_test_case_output_id!r}"
            )
        return cast(
            ApplicationTestCaseOutputRetrieveResponse,
            await self._get(
                f"/v4/application-test-case-outputs/{application_test_case_output_id}",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=await async_maybe_transform(
                        {"view": view},
                        application_test_case_output_retrieve_params.ApplicationTestCaseOutputRetrieveParams,
                    ),
                ),
                cast_to=cast(
                    Any, ApplicationTestCaseOutputRetrieveResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        application_variant_id: Union[int, str] | NotGiven = NOT_GIVEN,
        application_variant_report_id: Union[int, str] | NotGiven = NOT_GIVEN,
        evaluation_dataset_id: Union[int, str] | NotGiven = NOT_GIVEN,
        evaluation_dataset_version_num: Union[int, str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        view: List[Literal["MetricScores", "TestCaseVersion", "Trace"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[
        ApplicationTestCaseOutputListResponse, AsyncPageResponse[ApplicationTestCaseOutputListResponse]
    ]:
        """
        ### Description

        Lists all application test case outputs accessible to the user.

        ### Details

        This API can be used to list application test case outputs. If a user has access
        to multiple accounts, all application test case outputs from all accounts the
        user is associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/application-test-case-outputs",
            page=AsyncPageResponse[ApplicationTestCaseOutputListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "application_variant_id": application_variant_id,
                        "application_variant_report_id": application_variant_report_id,
                        "evaluation_dataset_id": evaluation_dataset_id,
                        "evaluation_dataset_version_num": evaluation_dataset_version_num,
                        "limit": limit,
                        "page": page,
                        "view": view,
                    },
                    application_test_case_output_list_params.ApplicationTestCaseOutputListParams,
                ),
            ),
            model=cast(
                Any, ApplicationTestCaseOutputListResponse
            ),  # Union types cannot be passed in as arguments in the type system
        )

    async def batch(
        self,
        *,
        items: Iterable[application_test_case_output_batch_params.Item],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationTestCaseOutputBatchResponse:
        """
        ### Description

        Creates a batch of application test case outputs

        ### Details

        This API can be used to create multiple application test case outputs so users
        do not have to the incur the cost of repeated network calls. To use this API,
        pass in a list of application test case outputs in the request body.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/application-test-case-outputs/batch",
            body=await async_maybe_transform(items, Iterable[application_test_case_output_batch_params.Item]),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationTestCaseOutputBatchResponse,
        )


class ApplicationTestCaseOutputsResourceWithRawResponse:
    def __init__(self, application_test_case_outputs: ApplicationTestCaseOutputsResource) -> None:
        self._application_test_case_outputs = application_test_case_outputs

        self.retrieve = to_raw_response_wrapper(
            application_test_case_outputs.retrieve,
        )
        self.list = to_raw_response_wrapper(
            application_test_case_outputs.list,
        )
        self.batch = to_raw_response_wrapper(
            application_test_case_outputs.batch,
        )


class AsyncApplicationTestCaseOutputsResourceWithRawResponse:
    def __init__(self, application_test_case_outputs: AsyncApplicationTestCaseOutputsResource) -> None:
        self._application_test_case_outputs = application_test_case_outputs

        self.retrieve = async_to_raw_response_wrapper(
            application_test_case_outputs.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            application_test_case_outputs.list,
        )
        self.batch = async_to_raw_response_wrapper(
            application_test_case_outputs.batch,
        )


class ApplicationTestCaseOutputsResourceWithStreamingResponse:
    def __init__(self, application_test_case_outputs: ApplicationTestCaseOutputsResource) -> None:
        self._application_test_case_outputs = application_test_case_outputs

        self.retrieve = to_streamed_response_wrapper(
            application_test_case_outputs.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            application_test_case_outputs.list,
        )
        self.batch = to_streamed_response_wrapper(
            application_test_case_outputs.batch,
        )


class AsyncApplicationTestCaseOutputsResourceWithStreamingResponse:
    def __init__(self, application_test_case_outputs: AsyncApplicationTestCaseOutputsResource) -> None:
        self._application_test_case_outputs = application_test_case_outputs

        self.retrieve = async_to_streamed_response_wrapper(
            application_test_case_outputs.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            application_test_case_outputs.list,
        )
        self.batch = async_to_streamed_response_wrapper(
            application_test_case_outputs.batch,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .usage_statistic_retrieve_params import UsageStatisticRetrieveParams as UsageStatisticRetrieveParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .annotation_config import AnnotationConfig

__all__ = ["Evaluation"]


class Evaluation(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    application_spec_id: str

    completed_test_case_result_count: int
    """The number of test case results that have been completed for the evaluation"""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    description: str

    name: str

    status: Literal["PENDING", "COMPLETED", "FAILED"]
    """An enumeration."""

    total_test_case_result_count: int
    """The total number of test case results for the evaluation"""

    annotation_config: Optional[AnnotationConfig] = None
    """Annotation configuration for tasking"""

    application_variant_id: Optional[str] = None

    archived_at: Optional[datetime] = None
    """The date and time when the entity was archived in ISO format."""

    completed_at: Optional[datetime] = None
    """
    The date and time that all test case results for the evaluation were completed
    for the evaluation in ISO format.
    """

    evaluation_config: Optional[object] = None

    evaluation_config_id: Optional[str] = None
    """The ID of the associated evaluation config."""

    question_id_to_annotation_config: Optional[Dict[str, AnnotationConfig]] = None
    """Specifies the annotation configuration to use for specific questions."""

    tags: Optional[object] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .application_configuration_param import ApplicationConfigurationParam

__all__ = [
    "ApplicationVariantCreateParams",
    "ApplicationVariantV0Request",
    "ApplicationVariantAgentsServiceRequest",
    "OfflineApplicationVariantRequest",
    "OfflineApplicationVariantRequestConfiguration",
]


class ApplicationVariantV0Request(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    configuration: Required[ApplicationConfigurationParam]

    name: Required[str]

    version: Required[Literal["V0"]]

    application_spec_id: str

    description: str
    """Optional description of the application variant"""


class ApplicationVariantAgentsServiceRequest(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    name: Required[str]

    version: Required[Literal["AGENTS_SERVICE"]]

    application_spec_id: str

    configuration: object

    description: str
    """Optional description of the application variant"""


class OfflineApplicationVariantRequest(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    configuration: Required[OfflineApplicationVariantRequestConfiguration]

    name: Required[str]

    version: Required[Literal["OFFLINE"]]

    application_spec_id: str

    description: str
    """Optional description of the application variant"""


class OfflineApplicationVariantRequestConfiguration(TypedDict, total=False):
    metadata: object
    """User defined metadata about the offline application"""

    output_schema_type: Literal["completion_only", "context_string", "context_chunks"]
    """An enumeration."""


ApplicationVariantCreateParams: TypeAlias = Union[
    ApplicationVariantV0Request, ApplicationVariantAgentsServiceRequest, OfflineApplicationVariantRequest
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["EmbeddingResponse"]


class EmbeddingResponse(BaseModel):
    embeddings: List[List[object]]

    tokens_used: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .feedback import (
    FeedbackResource,
    AsyncFeedbackResource,
    FeedbackResourceWithRawResponse,
    AsyncFeedbackResourceWithRawResponse,
    FeedbackResourceWithStreamingResponse,
    AsyncFeedbackResourceWithStreamingResponse,
)
from .messages import (
    MessagesResource,
    AsyncMessagesResource,
    MessagesResourceWithRawResponse,
    AsyncMessagesResourceWithRawResponse,
    MessagesResourceWithStreamingResponse,
    AsyncMessagesResourceWithStreamingResponse,
)

__all__ = [
    "FeedbackResource",
    "AsyncFeedbackResource",
    "FeedbackResourceWithRawResponse",
    "AsyncFeedbackResourceWithRawResponse",
    "FeedbackResourceWithStreamingResponse",
    "AsyncFeedbackResourceWithStreamingResponse",
    "MessagesResource",
    "AsyncMessagesResource",
    "MessagesResourceWithRawResponse",
    "AsyncMessagesResourceWithRawResponse",
    "MessagesResourceWithStreamingResponse",
    "AsyncMessagesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .messages import (
    MessagesResource,
    AsyncMessagesResource,
    MessagesResourceWithRawResponse,
    AsyncMessagesResourceWithRawResponse,
    MessagesResourceWithStreamingResponse,
    AsyncMessagesResourceWithStreamingResponse,
)
from .chat_threads import (
    ChatThreadsResource,
    AsyncChatThreadsResource,
    ChatThreadsResourceWithRawResponse,
    AsyncChatThreadsResourceWithRawResponse,
    ChatThreadsResourceWithStreamingResponse,
    AsyncChatThreadsResourceWithStreamingResponse,
)

__all__ = [
    "MessagesResource",
    "AsyncMessagesResource",
    "MessagesResourceWithRawResponse",
    "AsyncMessagesResourceWithRawResponse",
    "MessagesResourceWithStreamingResponse",
    "AsyncMessagesResourceWithStreamingResponse",
    "ChatThreadsResource",
    "AsyncChatThreadsResource",
    "ChatThreadsResourceWithRawResponse",
    "AsyncChatThreadsResourceWithRawResponse",
    "ChatThreadsResourceWithStreamingResponse",
    "AsyncChatThreadsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Artifact", "Chunk", "ChunksStatus"]


class Chunk(BaseModel):
    chunk_id: str
    """Unique ID of the chunk."""

    chunk_position: int
    """Position of the chunk in the artifact."""

    text: str
    """Associated text of the chunk."""

    upload_status: Literal["Embedding", "Pending", "Completed", "Failed", "Uploading"]
    """An enumeration."""

    embedding: Optional[List[float]] = None
    """Vector embedding of the chunk's text. Null if not embedded yet."""

    metadata: Optional[object] = None
    """Additional metadata associated with the chunk."""

    status_reason: Optional[str] = None
    """Reason for the chunk's upload status."""

    user_supplied_metadata: Optional[object] = None
    """Additional metadata associated with the chunk generated by custom chunking"""


class ChunksStatus(BaseModel):
    chunks_completed: int
    """Number of chunks uploaded successfully."""

    chunks_failed: int
    """Number of chunks that failed upload."""

    chunks_pending: int
    """Number of chunks awaiting upload."""


class Artifact(BaseModel):
    artifact_id: str
    """Unique identifier for the artifact."""

    artifact_name: str
    """Friendly name for the artifact."""

    artifact_uri: str
    """Location (e.g. URI) of the artifact in the data source."""

    chunks: List[Chunk]
    """List of chunks associated with the artifact."""

    chunks_status: ChunksStatus
    """Number of chunks pending, completed, and failed."""

    source: Literal[
        "S3", "Confluence", "SharePoint", "GoogleDrive", "AzureBlobStorage", "Slack", "LocalFile", "LocalChunks"
    ]
    """An enumeration."""

    status: str
    """Status of the artifact."""

    artifact_uri_public: Optional[str] = None
    """Public Location (e.g. URI) of the artifact in the data source."""

    status_reason: Optional[str] = None
    """Reason for the artifact's status."""

    updated_at: Optional[datetime] = None
    """Timestamp at which the artifact was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ApplicationSchemaRetrieveParams"]


class ApplicationSchemaRetrieveParams(TypedDict, total=False):
    version: Required[Literal["OFFLINE", "V0", "AGENTS_SERVICE"]]
    """
    An enum representing the version states of an application and its nodes'
    schemas. Attributes: V0: The initial version of an application schema.
    AGENTS_SERVICE: Application schema references egp_services definition.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Dict, Union, Iterable, cast, overload
from typing_extensions import Literal

import httpx

from ..types import (
    application_variant_list_params,
    application_variant_create_params,
    application_variant_process_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    required_args,
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncPageResponse, AsyncPageResponse
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.generic_delete_response import GenericDeleteResponse
from ..types.application_configuration_param import ApplicationConfigurationParam
from ..types.application_variant_list_response import ApplicationVariantListResponse
from ..types.application_variant_create_response import ApplicationVariantCreateResponse
from ..types.application_variant_retrieve_response import ApplicationVariantRetrieveResponse

__all__ = ["ApplicationVariantsResource", "AsyncApplicationVariantsResource"]


class ApplicationVariantsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ApplicationVariantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ApplicationVariantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ApplicationVariantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ApplicationVariantsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        account_id: str,
        configuration: ApplicationConfigurationParam,
        name: str,
        version: Literal["V0"],
        application_spec_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationVariantCreateResponse:
        """
        ### Description

        Creates a application variant

        ### Details

        This API can be used to create a application variant. To use this API, review
        the request schema and pass in all fields that are required to create a
        application variant.

        Args:
          account_id: The ID of the account that owns the given entity.

          description: Optional description of the application variant

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        account_id: str,
        name: str,
        version: Literal["AGENTS_SERVICE"],
        application_spec_id: str | NotGiven = NOT_GIVEN,
        configuration: object | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationVariantCreateResponse:
        """
        ### Description

        Creates a application variant

        ### Details

        This API can be used to create a application variant. To use this API, review
        the request schema and pass in all fields that are required to create a
        application variant.

        Args:
          account_id: The ID of the account that owns the given entity.

          description: Optional description of the application variant

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        account_id: str,
        configuration: application_variant_create_params.OfflineApplicationVariantRequestConfiguration,
        name: str,
        version: Literal["OFFLINE"],
        application_spec_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationVariantCreateResponse:
        """
        ### Description

        Creates a application variant

        ### Details

        This API can be used to create a application variant. To use this API, review
        the request schema and pass in all fields that are required to create a
        application variant.

        Args:
          account_id: The ID of the account that owns the given entity.

          description: Optional description of the application variant

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "configuration", "name", "version"], ["account_id", "name", "version"])
    def create(
        self,
        *,
        account_id: str,
        configuration: ApplicationConfigurationParam
        | object
        | application_variant_create_params.OfflineApplicationVariantRequestConfiguration
        | NotGiven = NOT_GIVEN,
        name: str,
        version: Literal["V0"] | Literal["AGENTS_SERVICE"] | Literal["OFFLINE"],
        application_spec_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationVariantCreateResponse:
        return cast(
            ApplicationVariantCreateResponse,
            self._post(
                "/v4/application-variants",
                body=maybe_transform(
                    {
                        "account_id": account_id,
                        "configuration": configuration,
                        "name": name,
                        "version": version,
                        "application_spec_id": application_spec_id,
                        "description": description,
                    },
                    application_variant_create_params.ApplicationVariantCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ApplicationVariantCreateResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def retrieve(
        self,
        application_variant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationVariantRetrieveResponse:
        """
        ### Description

        Gets the details of a application variant

        ### Details

        This API can be used to get information about a single application variant by
        ID. To use this API, pass in the `id` that was returned from your Create
        Application Variant API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_variant_id:
            raise ValueError(
                f"Expected a non-empty value for `application_variant_id` but received {application_variant_id!r}"
            )
        return cast(
            ApplicationVariantRetrieveResponse,
            self._get(
                f"/v4/application-variants/{application_variant_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ApplicationVariantRetrieveResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        application_spec_id: Union[int, str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[ApplicationVariantListResponse]:
        """
        ### Description

        Lists all application variants accessible to the user.

        ### Details

        This API can be used to list application variants. If a user has access to
        multiple accounts, all application variants from all accounts the user is
        associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/application-variants",
            page=SyncPageResponse[ApplicationVariantListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "application_spec_id": application_spec_id,
                        "limit": limit,
                        "page": page,
                    },
                    application_variant_list_params.ApplicationVariantListParams,
                ),
            ),
            model=cast(
                Any, ApplicationVariantListResponse
            ),  # Union types cannot be passed in as arguments in the type system
        )

    def delete(
        self,
        application_variant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a application variant

        ### Details

        This API can be used to delete a application variant by ID. To use this API,
        pass in the `id` that was returned from your Create Application Variant API call
        as a path parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_variant_id:
            raise ValueError(
                f"Expected a non-empty value for `application_variant_id` but received {application_variant_id!r}"
            )
        return self._delete(
            f"/v4/application-variants/{application_variant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )

    def process(
        self,
        application_variant_id: str,
        *,
        inputs: object,
        history: Iterable[application_variant_process_params.History] | NotGiven = NOT_GIVEN,
        overrides: Dict[str, application_variant_process_params.Overrides] | NotGiven = NOT_GIVEN,
        stream: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Process Application By Id

        Args:
          inputs: Input data for the application.

        You must provide inputs for each input node

          history: History of the application

          overrides: Optional overrides for the application

          stream: Control to have streaming of the endpoint. If the last node before the output is
              a completion node, you can set this to true to get the output as soon as the
              completion node has a token

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_variant_id:
            raise ValueError(
                f"Expected a non-empty value for `application_variant_id` but received {application_variant_id!r}"
            )
        return self._post(
            f"/v4/applications/{application_variant_id}/process",
            body=maybe_transform(
                {
                    "inputs": inputs,
                    "history": history,
                    "overrides": overrides,
                    "stream": stream,
                },
                application_variant_process_params.ApplicationVariantProcessParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncApplicationVariantsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncApplicationVariantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncApplicationVariantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncApplicationVariantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncApplicationVariantsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        account_id: str,
        configuration: ApplicationConfigurationParam,
        name: str,
        version: Literal["V0"],
        application_spec_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationVariantCreateResponse:
        """
        ### Description

        Creates a application variant

        ### Details

        This API can be used to create a application variant. To use this API, review
        the request schema and pass in all fields that are required to create a
        application variant.

        Args:
          account_id: The ID of the account that owns the given entity.

          description: Optional description of the application variant

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        account_id: str,
        name: str,
        version: Literal["AGENTS_SERVICE"],
        application_spec_id: str | NotGiven = NOT_GIVEN,
        configuration: object | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationVariantCreateResponse:
        """
        ### Description

        Creates a application variant

        ### Details

        This API can be used to create a application variant. To use this API, review
        the request schema and pass in all fields that are required to create a
        application variant.

        Args:
          account_id: The ID of the account that owns the given entity.

          description: Optional description of the application variant

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        account_id: str,
        configuration: application_variant_create_params.OfflineApplicationVariantRequestConfiguration,
        name: str,
        version: Literal["OFFLINE"],
        application_spec_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationVariantCreateResponse:
        """
        ### Description

        Creates a application variant

        ### Details

        This API can be used to create a application variant. To use this API, review
        the request schema and pass in all fields that are required to create a
        application variant.

        Args:
          account_id: The ID of the account that owns the given entity.

          description: Optional description of the application variant

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "configuration", "name", "version"], ["account_id", "name", "version"])
    async def create(
        self,
        *,
        account_id: str,
        configuration: ApplicationConfigurationParam
        | object
        | application_variant_create_params.OfflineApplicationVariantRequestConfiguration
        | NotGiven = NOT_GIVEN,
        name: str,
        version: Literal["V0"] | Literal["AGENTS_SERVICE"] | Literal["OFFLINE"],
        application_spec_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationVariantCreateResponse:
        return cast(
            ApplicationVariantCreateResponse,
            await self._post(
                "/v4/application-variants",
                body=await async_maybe_transform(
                    {
                        "account_id": account_id,
                        "configuration": configuration,
                        "name": name,
                        "version": version,
                        "application_spec_id": application_spec_id,
                        "description": description,
                    },
                    application_variant_create_params.ApplicationVariantCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ApplicationVariantCreateResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def retrieve(
        self,
        application_variant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationVariantRetrieveResponse:
        """
        ### Description

        Gets the details of a application variant

        ### Details

        This API can be used to get information about a single application variant by
        ID. To use this API, pass in the `id` that was returned from your Create
        Application Variant API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_variant_id:
            raise ValueError(
                f"Expected a non-empty value for `application_variant_id` but received {application_variant_id!r}"
            )
        return cast(
            ApplicationVariantRetrieveResponse,
            await self._get(
                f"/v4/application-variants/{application_variant_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ApplicationVariantRetrieveResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        application_spec_id: Union[int, str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ApplicationVariantListResponse, AsyncPageResponse[ApplicationVariantListResponse]]:
        """
        ### Description

        Lists all application variants accessible to the user.

        ### Details

        This API can be used to list application variants. If a user has access to
        multiple accounts, all application variants from all accounts the user is
        associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/application-variants",
            page=AsyncPageResponse[ApplicationVariantListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "application_spec_id": application_spec_id,
                        "limit": limit,
                        "page": page,
                    },
                    application_variant_list_params.ApplicationVariantListParams,
                ),
            ),
            model=cast(
                Any, ApplicationVariantListResponse
            ),  # Union types cannot be passed in as arguments in the type system
        )

    async def delete(
        self,
        application_variant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a application variant

        ### Details

        This API can be used to delete a application variant by ID. To use this API,
        pass in the `id` that was returned from your Create Application Variant API call
        as a path parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_variant_id:
            raise ValueError(
                f"Expected a non-empty value for `application_variant_id` but received {application_variant_id!r}"
            )
        return await self._delete(
            f"/v4/application-variants/{application_variant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )

    async def process(
        self,
        application_variant_id: str,
        *,
        inputs: object,
        history: Iterable[application_variant_process_params.History] | NotGiven = NOT_GIVEN,
        overrides: Dict[str, application_variant_process_params.Overrides] | NotGiven = NOT_GIVEN,
        stream: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Process Application By Id

        Args:
          inputs: Input data for the application.

        You must provide inputs for each input node

          history: History of the application

          overrides: Optional overrides for the application

          stream: Control to have streaming of the endpoint. If the last node before the output is
              a completion node, you can set this to true to get the output as soon as the
              completion node has a token

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_variant_id:
            raise ValueError(
                f"Expected a non-empty value for `application_variant_id` but received {application_variant_id!r}"
            )
        return await self._post(
            f"/v4/applications/{application_variant_id}/process",
            body=await async_maybe_transform(
                {
                    "inputs": inputs,
                    "history": history,
                    "overrides": overrides,
                    "stream": stream,
                },
                application_variant_process_params.ApplicationVariantProcessParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class ApplicationVariantsResourceWithRawResponse:
    def __init__(self, application_variants: ApplicationVariantsResource) -> None:
        self._application_variants = application_variants

        self.create = to_raw_response_wrapper(
            application_variants.create,
        )
        self.retrieve = to_raw_response_wrapper(
            application_variants.retrieve,
        )
        self.list = to_raw_response_wrapper(
            application_variants.list,
        )
        self.delete = to_raw_response_wrapper(
            application_variants.delete,
        )
        self.process = to_raw_response_wrapper(
            application_variants.process,
        )


class AsyncApplicationVariantsResourceWithRawResponse:
    def __init__(self, application_variants: AsyncApplicationVariantsResource) -> None:
        self._application_variants = application_variants

        self.create = async_to_raw_response_wrapper(
            application_variants.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            application_variants.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            application_variants.list,
        )
        self.delete = async_to_raw_response_wrapper(
            application_variants.delete,
        )
        self.process = async_to_raw_response_wrapper(
            application_variants.process,
        )


class ApplicationVariantsResourceWithStreamingResponse:
    def __init__(self, application_variants: ApplicationVariantsResource) -> None:
        self._application_variants = application_variants

        self.create = to_streamed_response_wrapper(
            application_variants.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            application_variants.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            application_variants.list,
        )
        self.delete = to_streamed_response_wrapper(
            application_variants.delete,
        )
        self.process = to_streamed_response_wrapper(
            application_variants.process,
        )


class AsyncApplicationVariantsResourceWithStreamingResponse:
    def __init__(self, application_variants: AsyncApplicationVariantsResource) -> None:
        self._application_variants = application_variants

        self.create = async_to_streamed_response_wrapper(
            application_variants.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            application_variants.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            application_variants.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            application_variants.delete,
        )
        self.process = async_to_streamed_response_wrapper(
            application_variants.process,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from ..s3_data_source_config import S3DataSourceConfig
from ..local_file_source_config import LocalFileSourceConfig
from ..slack_data_source_config import SlackDataSourceConfig
from ..local_chunks_source_config import LocalChunksSourceConfig
from ..confluence_data_source_config import ConfluenceDataSourceConfig
from ..share_point_data_source_config import SharePointDataSourceConfig
from ..token_chunking_strategy_config import TokenChunkingStrategyConfig
from ..custom_chunking_strategy_config import CustomChunkingStrategyConfig
from ..google_drive_data_source_config import GoogleDriveDataSourceConfig
from ..character_chunking_strategy_config import CharacterChunkingStrategyConfig
from ..azure_blob_storage_data_source_config import AzureBlobStorageDataSourceConfig

__all__ = ["UploadListResponse", "DataSourceConfig", "ChunkingStrategyConfig"]

DataSourceConfig: TypeAlias = Annotated[
    Union[
        S3DataSourceConfig,
        SharePointDataSourceConfig,
        GoogleDriveDataSourceConfig,
        AzureBlobStorageDataSourceConfig,
        LocalChunksSourceConfig,
        LocalFileSourceConfig,
        ConfluenceDataSourceConfig,
        SlackDataSourceConfig,
    ],
    PropertyInfo(discriminator="source"),
]

ChunkingStrategyConfig: TypeAlias = Annotated[
    Union[CharacterChunkingStrategyConfig, TokenChunkingStrategyConfig, CustomChunkingStrategyConfig],
    PropertyInfo(discriminator="strategy"),
]


class UploadListResponse(BaseModel):
    id: str

    data_source_config: DataSourceConfig

    knowledge_base_id: str

    chunking_strategy_config: Optional[ChunkingStrategyConfig] = None

    created_at: Union[str, datetime, None] = None

    created_by_schedule_id: Optional[str] = None

    data_source_idempotency_key: Optional[str] = None

    status: Optional[str] = None

    status_reason: Optional[str] = None

    updated_at: Union[str, datetime, None] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

__all__ = ["DeploymentListParams"]


class DeploymentListParams(TypedDict, total=False):
    account_id: str
    """Optional filter by account id"""

    limit: int
    """Maximum number of artifacts to be returned by the given endpoint.

    Defaults to 100 and cannot be greater than 10k.
    """

    page: int
    """Page number for pagination to be returned by the given endpoint.

    Starts at page 1
    """

    sort_by: List[
        Literal[
            "model_creation_parameters:asc",
            "model_creation_parameters:desc",
            "model_endpoint_id:asc",
            "model_endpoint_id:desc",
            "model_instance_id:asc",
            "model_instance_id:desc",
            "vendor_configuration:asc",
            "vendor_configuration:desc",
            "deployment_metadata:asc",
            "deployment_metadata:desc",
            "status:asc",
            "status:desc",
            "id:asc",
            "id:desc",
            "created_at:asc",
            "created_at:desc",
            "account_id:asc",
            "account_id:desc",
            "created_by_user_id:asc",
            "created_by_user_id:desc",
            "created_by_user:asc",
            "created_by_user:desc",
            "name:asc",
            "name:desc",
        ]
    ]
